import requests
import base64
import base58

from hm_pyhelper.protos import blockchain_txn_pb2, \
                               blockchain_txn_add_gateway_v1_pb2
from hm_pyhelper.miner_json_rpc.exceptions import MinerConnectionError, \
                                                  MinerMalformedURL, \
                                                  MinerRegionUnset, \
                                                  MinerMalformedAddGatewayTxn


class Client(object):

    def __init__(self, url='http://helium-miner:4467'):
        self.url = url

    def __fetch_data(self, method, **kwargs):
        req_body = {
            "jsonrpc": "2.0",
            "id": 1,
            "method": method,
        }
        if kwargs:
            req_body["params"] = kwargs
        try:
            response = requests.post(self.url, json=req_body)
        except requests.exceptions.ConnectionError:
            raise MinerConnectionError(
                "Unable to connect to miner %s" % self.url
            )
        except requests.exceptions.MissingSchema:
            raise MinerMalformedURL(
                "Miner JSONRPC URL '%s' is not a valid URL"
                % self.url
            )

        if not response.ok:
            response.raise_for_status()

        return response.json().get('result')

    def get_height(self):
        return self.__fetch_data('info_height')

    def get_region(self):
        region = self.__fetch_data('info_region')
        if not region.get('region'):
            raise MinerRegionUnset(
                "Miner at %s does not have an asserted region"
                % self.url
            )
        return region

    def get_summary(self):
        return self.__fetch_data('info_summary')

    def get_peer_addr(self):
        return self.__fetch_data('peer_addr')

    def get_peer_book(self):
        return self.__fetch_data('peer_book', addr='self')

    def get_firmware_version(self):
        summary = self.get_summary()
        return summary.get('firmware_version')

    def create_add_gateway_txn(self, owner_address: str, payer_address: str,
                               gateway_address: str = None) -> dict:
        """
        Invokes the txn_add_gateway RPC endpoint on the miner and returns
        the same payload that the smartphone app traditionally expects.
        https://docs.helium.com/mine-hnt/full-hotspots/become-a-maker/hotspot-integration-testing/#generate-an-add-hotspot-transaction

        Alternatively, the same thing can be accomplished with dbus,
        like in the below, but RPC is generally easier to use.
        https://github.com/NebraLtd/hm-config/blob/900aeed353fb9729b49bca97d7da8a9abf0a2029/gatewayconfig/bluetooth/characteristics/add_gateway_characteristic.py#L71

        Parameters:
            - owner_address: The address of the account that owns the gateway.
            - payer_address: The address of the account that will pay for the
                             transaction. This will typically be the
                             maker/Nebra's account.
            - gateway_address: The address of the miner itself. This is
                               an optional parameter because the miner
                               will always return it in the payload during
                               transaction generation. If the param is
                               provided, it will only be used as extra
                               validation.

        Raises:
            - MinerMalformedAddGatewayTxn if returned transaction does
              not correspond to the supplied parameters.
        """
        # Invoke add_gateway_txn on miner
        # https://github.com/helium/miner/blob/b9d2cd108cdcc864b641ccf4209f790b1461926d/src/jsonrpc/miner_jsonrpc_txn.erl#L31
        rpc_response = self.__fetch_data('txn_add_gateway',
                                         owner=owner_address,
                                         payer=payer_address)
        encoded_wrapped_txn = rpc_response['result']

        # Base64 decode
        # https://github.com/helium/miner/blob/b9d2cd108cdcc864b641ccf4209f790b1461926d/src/jsonrpc/miner_jsonrpc_txn.erl#L38
        decoded_wrapped_txn = base64.b64decode(encoded_wrapped_txn)

        # Deserialize wrapped protobuf
        # https://github.com/helium/blockchain-core/blob/3cd6bca6c5595a1363a9bbd625ef254383a4141b/src/transactions/blockchain_txn.erl#L160
        wrapped_txn = blockchain_txn_pb2.blockchain_txn()
        wrapped_txn.ParseFromString(decoded_wrapped_txn)

        # Unwrap to get blockchain_txn_add_gateway_v1
        # https://github.com/helium/proto/blob/6dc60a9933628c3baf9d2f5386481f20a5d79bb8/src/blockchain_txn_add_gateway_v1.proto#L1
        add_gateway_txn = wrapped_txn.add_gateway

        txn_gateway_address = get_address_from_add_gateway_txn(
            add_gateway_txn, 'gateway', gateway_address)

        txn_owner_address = get_address_from_add_gateway_txn(
            add_gateway_txn, 'owner', owner_address)

        txn_payer_address = get_address_from_add_gateway_txn(
            add_gateway_txn, 'payer', payer_address)

        return {
            'gateway_address': txn_gateway_address,
            'owner_address': txn_owner_address,
            'payer_address': txn_payer_address,
            'fee': add_gateway_txn.fee,
            'staking_fee': add_gateway_txn.staking_fee,
            'txn': encoded_wrapped_txn
        }


def get_address_from_add_gateway_txn(add_gateway_txn:
                                     blockchain_txn_add_gateway_v1_pb2,
                                     address_type: str,
                                     expected_address: str = None):
    """
    Deserializes specified field in the blockchain_txn_add_gateway_v1_pb2
    protobuf to a base58 Helium address.

    Pararms:
        - add_gateway_txn: The blockchain_txn_add_gateway_v1_pb2 to
                           inspect.
        - address_type: 'owner', 'gateway', or 'payer'.
        - expected_address (optional): Value we expect to be returned.

    Raises:
        MinerMalformedAddGatewayTxn if expected_address supplied and
        does not match the return value.
    """

    # Addresses returned by the RPC response are missing a leading
    # byte for the version. The version is currently always 0.
    # https://github.com/helium/helium-js/blob/8d5cb76e156fb80de6fc80f239b43e3872c7b7d7/packages/crypto/src/Address.ts#L64
    version_byte = b'\x00'

    # Convert binary address to base58
    address_bytes = version_byte + getattr(add_gateway_txn, address_type)
    address = str(base58.b58encode_check(address_bytes), 'utf-8')

    # Ensure resulting address matches expectation
    is_expected_address_defined = expected_address is not None
    if is_expected_address_defined and address != expected_address:
        msg = f"Expected {address_type} address to be {expected_address}," + \
              f"but is {address}"
        raise MinerMalformedAddGatewayTxn(msg)

    return address



import json
import unittest
import pytest
from unittest.mock import ANY, mock_open, patch
from packaging.version import Version
from hm_pyhelper.exceptions import ECCMalfunctionException, \
    MinerFailedToFetchMacAddress, GatewayMFRInvalidVersion, GatewayMFRExecutionException, \
    GatewayMFRFileNotFoundException, UnsupportedGatewayMfrVersion
from hm_pyhelper.lock_singleton import ResourceBusyError
from hm_pyhelper.miner_param import retry_get_region, await_spi_available, \
    provision_key, run_gateway_mfr, \
    did_gateway_mfr_test_result_include_miner_key_pass, \
    get_mac_address, get_public_keys_rust, get_gateway_mfr_version, get_gateway_mfr_command


ALL_PASS_GATEWAY_MFR_TESTS = {
    'ecdh(0)': {'error': 'decode error\n\nCaused by:\n    not a compact key', 'result': 'fail'},
    'key_config(0)': {
        'checks': {
            'auth_key': '0',
            'intrusion_disable': 'false',
            'key_type': 'ecc',
            'lockable': 'true',
            'private': 'true',
            'pub_info': 'true',
            'req_auth': 'false',
            'req_random': 'false',
            'x509_index': '0'
        },
        'result': 'pass'
    },
    'miner_key(0)': {'checks': 'ok', 'result': 'pass'},
    'sign(0)': {'checks': 'ok', 'result': 'pass'},
    'slot_config(0)': {
        'checks': {
            'ecdh_operation': 'true',
            'encrypt_read': 'false',
            'external_signatures': 'true',
            'internal_signatures': 'true',
            'limited_use': 'false',
            'secret': 'true'
        },
        'result': 'pass'
    },
    'zone_locked(config)': {'checks': 'ok', 'result': 'pass'},
    'zone_locked(data)': {'checks': 'ok', 'result': 'pass'}
}

ERROR_MESSAGE = 'decode error\n\nCaused by:\n    not a compact key'

NONE_PASS_GATEWAY_MFR_TESTS = {
    'ecdh(0)': {'error': ERROR_MESSAGE, 'result': 'fail'},
    'key_config(0)': {'error': ERROR_MESSAGE, 'result': 'fail'},
    'miner_key(0)': {'error': ERROR_MESSAGE, 'result': 'fail'},
    'sign(0)': {'error': ERROR_MESSAGE, 'result': 'fail'},
    'slot_config(0)': {'error': ERROR_MESSAGE, 'result': 'fail'},
    'zone_locked(config)': {'error': ERROR_MESSAGE, 'result': 'fail'},
    'zone_locked(data)': {'error': ERROR_MESSAGE, 'result': 'fail'}
}

MOCK_VARIANT_DEFINITIONS = {
        'NEBHNT-WITH-ECC-ADDRESS': {
            'KEY_STORAGE_BUS': '/dev/i2c-X',
            'SWARM_KEY_URI': 'ecc://i2c-X:96?slot=0',
        },
        'NEBHNT-NO-ECC-ADDRESS': {
            'NO_KEY_STORAGE_BUS': '/dev/i2c-X',
            'NO_KEY_SWARM_KEY_URI': 'ecc://i2c-X:96?slot=0',
        }
    }


class SubprocessResult(object):
    def __init__(self, stdout=None, stderr=None):
        self.stdout = stdout
        self.stderr = stderr


@patch.dict('os.environ', {"VARIANT": "NEBHNT-WITH-ECC-ADDRESS"})
@patch('hm_pyhelper.hardware_definitions.variant_definitions',
       MOCK_VARIANT_DEFINITIONS)
class TestMinerParam(unittest.TestCase):

    @patch('subprocess.run', side_effect=FileNotFoundError("File Not Found Error"))
    def test_get_gateway_mfr_version_exception(self, mocked_subprocess_run):
        with self.assertRaises(GatewayMFRExecutionException):
            get_gateway_mfr_version()

        mocked_subprocess_run.assert_called_once_with(
            [ANY, '--version'], capture_output=True, check=True)

    @patch('subprocess.run', return_value=SubprocessResult(stdout='invalid version'))
    def test_get_gateway_mfr_version_invalid_version(self, mocked_subprocess_run):
        with self.assertRaises(GatewayMFRInvalidVersion):
            get_gateway_mfr_version()

        mocked_subprocess_run.assert_called_once_with(
            [ANY, '--version'], capture_output=True, check=True)

    @patch('subprocess.run', return_value=SubprocessResult(stdout=b'gateway_mfr 0.1.7'))
    def test_get_gateway_mfr_version_v017(self, mocked_subprocess_run):
        version = get_gateway_mfr_version()

        self.assertEqual(version.major, 0)
        self.assertEqual(version.minor, 1)
        self.assertEqual(version.micro, 7)

        mocked_subprocess_run.assert_called_once_with(
            [ANY, '--version'], capture_output=True, check=True)

    @patch('subprocess.run', return_value=SubprocessResult(stdout=b'gateway_mfr 0.2.1'))
    def test_get_gateway_mfr_version_v021(self, mocked_subprocess_run):
        version = get_gateway_mfr_version()

        self.assertEqual(version.major, 0)
        self.assertEqual(version.minor, 2)
        self.assertEqual(version.micro, 1)

        mocked_subprocess_run.assert_called_once_with(
            [ANY, '--version'], capture_output=True, check=True)

    @patch('hm_pyhelper.miner_param.get_gateway_mfr_version',
           return_value=Version('0.1.7'))
    def test_get_gateway_mfr_command_v017(self, mocked_get_gateway_mfr_version):
        actual_result = get_gateway_mfr_command('key')
        expected_result = [ANY, '--path', '/dev/i2c-X', 'key', '0']
        self.assertListEqual(actual_result, expected_result)
        mocked_get_gateway_mfr_version.assert_called_once()

        actual_result = get_gateway_mfr_command('info')
        expected_result = [ANY, '--path', '/dev/i2c-X', 'info']
        self.assertListEqual(actual_result, expected_result)

    @patch.dict('os.environ', {"VARIANT": "NEBHNT-INVALID"})
    @patch('hm_pyhelper.miner_param.get_gateway_mfr_version',
           return_value=Version('0.1.7'))
    def test_get_gateway_mfr_command_v017_no_variant(self, mocked_get_gateway_mfr_version):
        actual_result = get_gateway_mfr_command('key')
        expected_result = [ANY, 'key', '0']
        self.assertListEqual(actual_result, expected_result)
        mocked_get_gateway_mfr_version.assert_called_once()

        actual_result = get_gateway_mfr_command('info')
        expected_result = [ANY, 'info']
        self.assertListEqual(actual_result, expected_result)

    @patch.dict('os.environ', {"VARIANT": "NEBHNT-NO-ECC-ADDRESS"})
    @patch('hm_pyhelper.miner_param.get_gateway_mfr_version',
           return_value=Version('0.1.7'))
    def test_get_gateway_mfr_command_v017_no_KEY_STORAGE_BUS(self, mocked_get_gateway_mfr_version):
        actual_result = get_gateway_mfr_command('key')
        expected_result = [ANY, 'key', '0']
        self.assertListEqual(actual_result, expected_result)
        mocked_get_gateway_mfr_version.assert_called_once()

        actual_result = get_gateway_mfr_command('info')
        expected_result = [ANY, 'info']
        self.assertListEqual(actual_result, expected_result)

    @patch('hm_pyhelper.miner_param.get_gateway_mfr_version',
           return_value=Version('0.2.1'))
    def test_get_gateway_mfr_command_v021(self, mocked_get_gateway_mfr_version):
        actual_result = get_gateway_mfr_command('key')
        expected_result = [ANY, '--device', 'ecc://i2c-X:96?slot=0', 'key']
        self.assertListEqual(actual_result, expected_result)
        mocked_get_gateway_mfr_version.assert_called_once()

        actual_result = get_gateway_mfr_command('info')
        expected_result = [ANY, '--device', 'ecc://i2c-X:96?slot=0', 'info']
        self.assertListEqual(actual_result, expected_result)

    @patch.dict('os.environ', {"VARIANT": "NEBHNT-INVALID"})
    @patch('hm_pyhelper.miner_param.get_gateway_mfr_version',
           return_value=Version('0.2.1'))
    def test_get_gateway_mfr_command_v021_no_variant(self, mocked_get_gateway_mfr_version):
        actual_result = get_gateway_mfr_command('key')
        expected_result = [ANY, 'key']
        self.assertListEqual(actual_result, expected_result)
        mocked_get_gateway_mfr_version.assert_called_once()

        actual_result = get_gateway_mfr_command('test')
        expected_result = [ANY, 'test']
        self.assertListEqual(actual_result, expected_result)

    @patch.dict('os.environ', {"VARIANT": "NEBHNT-NO-ECC-ADDRESS"})
    @patch('hm_pyhelper.miner_param.get_gateway_mfr_version',
           return_value=Version('0.2.1'))
    def test_get_gateway_mfr_command_v021_no_SWARM_KEY_URI(self, mocked_get_gateway_mfr_version):
        actual_result = get_gateway_mfr_command('key')
        expected_result = [ANY, 'key']
        self.assertListEqual(actual_result, expected_result)
        mocked_get_gateway_mfr_version.assert_called_once()

        actual_result = get_gateway_mfr_command('test')
        expected_result = [ANY, 'test']
        self.assertListEqual(actual_result, expected_result)

    @patch('hm_pyhelper.miner_param.get_gateway_mfr_version',
           return_value=Version('0.2.0'))
    def test_get_gateway_mfr_command_v020(self, mocked_get_gateway_mfr_version):
        actual_result = get_gateway_mfr_command('key')
        expected_result = [ANY, '--device', 'ecc://i2c-X:96?slot=0', 'key']
        self.assertListEqual(actual_result, expected_result)
        mocked_get_gateway_mfr_version.assert_called_once()

        actual_result = get_gateway_mfr_command('info')
        expected_result = [ANY, '--device', 'ecc://i2c-X:96?slot=0', 'info']

    @patch('hm_pyhelper.miner_param.get_gateway_mfr_version',
           return_value=Version('0.3.9'))
    def test_get_gateway_mfr_command_v021_upward(self, mocked_get_gateway_mfr_version):
        actual_result = get_gateway_mfr_command('key')
        expected_result = [ANY, '--device', 'ecc://i2c-X:96?slot=0', 'key']
        self.assertListEqual(actual_result, expected_result)
        mocked_get_gateway_mfr_version.assert_called_once()

        actual_result = get_gateway_mfr_command('info')
        expected_result = [ANY, '--device', 'ecc://i2c-X:96?slot=0', 'info']

    @patch('hm_pyhelper.miner_param.get_gateway_mfr_version',
           return_value=Version('0.0.0'))
    def test_get_gateway_mfr_command_unsupported_version(self, mocked_get_gateway_mfr_version):
        with self.assertRaises(UnsupportedGatewayMfrVersion):
            get_gateway_mfr_command('key')
        mocked_get_gateway_mfr_version.assert_called_once()

    @patch('hm_pyhelper.miner_param.get_gateway_mfr_command',
           return_value=['gateway_mfr', 'arg1', 'arg2'])
    @patch('subprocess.run', side_effect=FileNotFoundError())
    def test_run_gateway_mfr_GatewayMFRFileNotFoundException(
            self,
            mocked_subprocess_run,
            mocked_get_gateway_mfr_command
    ):
        with self.assertRaises(GatewayMFRFileNotFoundException):
            run_gateway_mfr("unittest")

        mocked_get_gateway_mfr_command.assert_called_once_with('unittest', slot=0)
        mocked_subprocess_run.assert_called_once_with(
            ['gateway_mfr', 'arg1', 'arg2'],
            capture_output=True, check=True)

    @patch('hm_pyhelper.miner_param.get_gateway_mfr_command',
           return_value=['gateway_mfr', 'arg1', 'arg2'])
    @patch('subprocess.run', side_effect=Exception())
    def test_run_gateway_mfr_ECCMalfunctionException(
            self,
            mocked_subprocess_run,
            mocked_get_gateway_mfr_command
    ):
        with self.assertRaises(ECCMalfunctionException):
            run_gateway_mfr("unittest")

        mocked_get_gateway_mfr_command.assert_called_once_with('unittest', slot=0)
        mocked_subprocess_run.assert_called_once_with(
            ['gateway_mfr', 'arg1', 'arg2'],
            capture_output=True, check=True)

    @patch('hm_pyhelper.miner_param.get_gateway_mfr_command',
           return_value=['gateway_mfr', 'arg1', 'arg2'])
    @patch('subprocess.run', side_effect=ResourceBusyError())
    def test_run_gateway_mfr_ResourceBusyError(
            self,
            mocked_subprocess_run,
            mocked_get_gateway_mfr_command
    ):
        with self.assertRaises(ResourceBusyError):
            run_gateway_mfr("unittest")

        mocked_get_gateway_mfr_command.assert_called_once_with('unittest', slot=0)
        mocked_subprocess_run.assert_called_once_with(
            ['gateway_mfr', 'arg1', 'arg2'],
            capture_output=True, check=True)

    @patch('hm_pyhelper.miner_param.get_gateway_mfr_command',
           return_value=['gateway_mfr', 'arg1', 'arg2'])
    @patch('subprocess.run',
           return_value=SubprocessResult(
               stdout=json.dumps({
                   "key": "ABCD123456789",
                   "name": "formal-magenta-anteater",
                   "slot": 0
               })
           ))
    def test_get_public_keys_rust(
            self,
            mocked_subprocess_run,
            mocked_get_gateway_mfr_command
    ):
        actual_result = get_public_keys_rust()
        expected_result = {
            "key": "ABCD123456789",
            "name": "formal-magenta-anteater",
            "slot": 0
        }

        self.assertDictEqual(actual_result, expected_result)

        mocked_get_gateway_mfr_command.assert_called_once_with('key', slot=0)
        mocked_subprocess_run.assert_called_once_with(
            ['gateway_mfr', 'arg1', 'arg2'],
            capture_output=True, check=True)

    def test_provision_key_all_passed(self):
        self.assertTrue(provision_key(slot=0))

    @patch('hm_pyhelper.miner_param.run_gateway_mfr')
    def test_provision_key_none_passed(self, mocked_run_gateway_mfr):
        self.assertTrue(provision_key(slot=0))

        mocked_run_gateway_mfr.assert_called_once()

    @patch(
            'hm_pyhelper.miner_param.get_gateway_mfr_test_result',
            return_value={
                "result": "fail",
                "tests": NONE_PASS_GATEWAY_MFR_TESTS
            }
    )
    def test_did_gateway_mfr_test_result_include_miner_key_fail(
            self,
            get_gateway_mfr_test_result
    ):
        self.assertFalse(
                did_gateway_mfr_test_result_include_miner_key_pass(
                    get_gateway_mfr_test_result
                    )
        )

    def test_did_gateway_mfr_test_result_include_miner_key_pass(self):
        get_gateway_mfr_test_result = {
            "result": "fail",
            "tests": ALL_PASS_GATEWAY_MFR_TESTS
        }
        self.assertTrue(
                did_gateway_mfr_test_result_include_miner_key_pass(
                    get_gateway_mfr_test_result
                    )
        )

    def test_get_region_from_override(self):
        self.assertEqual(retry_get_region("foo", "bar/"), 'foo')

    @patch("builtins.open", new_callable=mock_open, read_data="ZZ111\n")
    def test_get_region_from_miner(self, _):
        self.assertEqual(retry_get_region(False, "foo/"), 'ZZ111')  # noqa: E501

    @patch("os.path.exists", return_value=True)
    def test_is_spi_available(self, _):
        self.assertTrue(await_spi_available("spiXY.Z"))

    def test_error_mac_address(self):
        with pytest.raises(MinerFailedToFetchMacAddress):
            get_mac_address("test/path")



import hm_pyhelper.sbc as sbc
from hm_pyhelper.exceptions import UnknownVariantException, \
    UnknownVariantAttributeException


def is_rockpi() -> bool:
    return sbc.is_sbc_type(sbc.DeviceVendorID.ROCK_PI)


def is_raspberry_pi() -> bool:
    return sbc.is_sbc_type(sbc.DeviceVendorID.RASPBERRY_PI)


# The variant name is determined by following rules:
# - Format : <vendor>-<model>
# - All lower case.
# - Models are named fl1,fl2...fln, based on release chronology.
# - Any new models should be added to the end of the list and follow this naming convention.
# - Three are some names here that don't follow this convention.
#   These historical names are all capital letters and we don't consider them in use.
variant_definitions = {
    # Nebra Indoor Hotspot Gen1
    'nebra-indoor1': {
        'FRIENDLY': 'Nebra Indoor Hotspot Gen 1',
        'SUPPORTED_MODELS': ['Nebra Indoor Hotspot Gen 1'],
        'CPU_ARCH': 'aarch64',
        'BALENA_DEVICE_TYPE': ['raspberrypi3-64'],
        'SPIBUS': 'spidev1.2',
        'KEY_STORAGE_BUS': '/dev/i2c-1',
        'SWARM_KEY_URI': 'ecc://i2c-1:96?slot=0',
        'RESET': 38,
        'MAC': 'eth0',
        'STATUS': 25,
        'BUTTON': 26,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False,
        'FCC_IDS': ['2AZDM-HNTIN'],
        'CONTAINS_FCC_IDS': ['2AHRD-EPN8531', '2AB8JCSR40', '2ARPP-GL5712UX'],
        'IC_IDS': ['27187-HNTIN'],
        'CONTAINS_IC_IDS': []
        },

    # Nebra Outdoor Hotspot Gen1
    'nebra-outdoor1': {
        'FRIENDLY': 'Nebra Outdoor Hotspot Gen 1',
        'SUPPORTED_MODELS': ['Nebra Outdoor Hotspot Gen 1'],
        'CPU_ARCH': 'aarch64',
        'BALENA_DEVICE_TYPE': ['raspberrypi3-64'],
        'SPIBUS': 'spidev1.2',
        'KEY_STORAGE_BUS': '/dev/i2c-1',
        'SWARM_KEY_URI': 'ecc://i2c-1:96?slot=0',
        'RESET': 38,
        'MAC': 'eth0',
        'STATUS': 25,
        'BUTTON': 24,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': True,
        'FCC_IDS': ['2AZDM-HNTOUT'],
        'CONTAINS_FCC_IDS': ['2ARPP-GL5712UX', '2AZDM-CSR8510',
                             'XMR201903EG25G', '2AZDM-WIFIRP'],
        'IC_IDS': ['27187-HNTOUT'],
        'CONTAINS_IC_IDS': []
        },

    # Nebra Outdoor Hotspot Gen2
    # Note:: assuming outdoor2 is equivalent to light miner 2
    'nebra-outdoor2': {
        'FRIENDLY': 'Nebra ROCK Pi 4 Outdoor',
        'SUPPORTED_MODELS': ['Nebra ROCK Pi 4 Outdoor'],
        'CPU_ARCH': 'aarch64',
        'BALENA_DEVICE_TYPE': ['rockpi-4b-rk3399'],
        'SPIBUS': 'spidev32766.0',
        'KEY_STORAGE_BUS': '/dev/i2c-7',
        'SWARM_KEY_URI': 'ecc://i2c-7:96?slot=0',
        'RESET': 149,
        'MAC': 'eth0',
        'STATUS': 156,
        'BUTTON': 154,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': True,
        'GPIO_PIN_LED': 18,
        'GPIO_PIN_BUTTON': 16,
        'FCC_IDS': ['2AZDM-HHRK4-OUT'],
        'CONTAINS_FCC_IDS': ['2ARPP-GL5712UX',
                             '2A3PA-ROCKPI4',
                             'XMR201903EG25G'],
        'IC_IDS': ['27187-HHRK4-OUT'],
        'CONTAINS_IC_IDS': []
        },

    # Nebra Pi 0 Light Hotspot SPI Ethernet
    'nebra-light1': {
        'FRIENDLY': 'Nebra Pi 0 Light Hotspot SE',
        'SUPPORTED_MODELS': ['Nebra Pi 0 Light Hotspot SE'],
        'CPU_ARCH': 'armv6hf',
        'BALENA_DEVICE_TYPE': ['raspberry-pi'],
        'SPIBUS': 'spidev1.2',
        'KEY_STORAGE_BUS': '/dev/i2c-1',
        'SWARM_KEY_URI': 'ecc://i2c-1:96?slot=0',
        'RESET': 22,
        'MAC': 'wlan0',
        'STATUS': 24,
        'BUTTON': 23,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False,
        'FCC_IDS': ['2AZDM-HNTLGTMC'],
        'CONTAINS_FCC_IDS': ['2ABCB-RPI0W', '2ARPP-GL5712UX'],
        'IC_IDS': ['27187-HNTLGTMC'],
        'CONTAINS_IC_IDS': ['20953-RPI0W']
        },

    # Nebra Pi 0 Light Hotspot USB Ethernet
    'NEBHNT-LGT-ZX': {
        'FRIENDLY': 'Nebra Pi 0 Light Hotspot XE',
        'SUPPORTED_MODELS': ['Nebra Pi 0 Light Hotspot XE'],
        'CPU_ARCH': 'armv6hf',
        'BALENA_DEVICE_TYPE': ['raspberry-pi'],
        'SPIBUS': 'spidev1.2',
        'KEY_STORAGE_BUS': '/dev/i2c-1',
        'SWARM_KEY_URI': 'ecc://i2c-1:96?slot=0',
        'RESET': 22,
        'MAC': 'wlan0',
        'STATUS': 24,
        'BUTTON': 23,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False,
        'FCC_IDS': [],
        'CONTAINS_FCC_IDS': [],
        'IC_IDS': [],
        'CONTAINS_IC_IDS': []
        },

    # Nebra Radxa Zero Light Hotspot II SPI Ethernet
    'nebra-light2': {
        'FRIENDLY': 'Nebra Radxa Zero Light Hotspot II SE',
        'SUPPORTED_MODELS': ['Nebra Radxa Zero Light Hotspot II SE'],
        'CPU_ARCH': 'aarch64',
        'BALENA_DEVICE_TYPE': ['radxa-zero'],
        'SPIBUS': 'spidev0.0',
        'KEY_STORAGE_BUS': '/dev/i2c-3',
        'SWARM_KEY_URI': 'ecc://i2c-3:96?slot=0',
        'RESET': 415,
        'MAC': 'wlan0',
        'STATUS': 421,
        'BUTTON': 416,
        'ECCOB': True,
        'TYPE': 'Light',
        'CELLULAR': False,
        'FCC_IDS': ['2AZDM-HNTLGTMC-RADXA'],
        'CONTAINS_FCC_IDS': ['2A3PA-RADXA-ZERO', '2ARPP-GL5712UX'],
        'IC_IDS': ['27187-HNTLGTMC-RADXA'],
        'CONTAINS_IC_IDS': []
        },

    # Nebra Raspberry Pi Zero Light Hotspot II SPI Ethernet
    'nebra-light3': {
        'FRIENDLY': 'Nebra Raspberry Pi Zero Light Hotspot II SE',
        'SUPPORTED_MODELS': ['Nebra Raspberry Pi Zero Light Hotspot II SE'],
        'CPU_ARCH': 'armv6hf',
        'BALENA_DEVICE_TYPE': ['raspberry-pi'],
        'SPIBUS': 'spidev1.2',
        'KEY_STORAGE_BUS': '/dev/i2c-1',
        'RESET': 4,
        'MAC': 'wlan0',
        'STATUS': 26,
        'BUTTON': 12,
        'ECCOB': True,
        'TYPE': 'Light',
        'CELLULAR': False,
        'FCC_IDS': ['2AZDM-HNTLGTMC'],
        'CONTAINS_FCC_IDS': ['2ABCB-RPI0W', '2ARPP-GL5712UX'],
        'IC_IDS': ['27187-HNTLGTMC'],
        'CONTAINS_IC_IDS': ['20953-RPI0W']
        },

    # Nebra Beaglebone Light Hotspot
    'NEBHNT-BBB': {
        'FRIENDLY': 'Nebra Beaglebone Light Hotspot',
        'SUPPORTED_MODELS': ['Nebra Beaglebone Light Hotspot'],
        'CPU_ARCH': 'armv7hf',
        'BALENA_DEVICE_TYPE': ['beaglebone-black'],
        'SPIBUS': 'spidev1.0',
        'KEY_STORAGE_BUS': '/dev/i2c-1',
        'SWARM_KEY_URI': 'ecc://i2c-1:96?slot=0',
        'RESET': 60,
        'MAC': 'eth0',
        'STATUS': 31,
        'BUTTON': 30,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False,
        'FCC_IDS': [],
        'CONTAINS_FCC_IDS': [],
        'IC_IDS': [],
        'CONTAINS_IC_IDS': []
        },

    # Nebra Pocket Beagle Light Hotspot
    'NEBHNT-PBB': {
        'FRIENDLY': 'Nebra Pocket Beagle Light Hotspot',
        'SUPPORTED_MODELS': ['Nebra Pocket Beagle Light Hotspot'],
        'CPU_ARCH': 'armv7hf',
        'BALENA_DEVICE_TYPE': ['beaglebone-pocket'],
        'SPIBUS': 'spidev1.2',
        'KEY_STORAGE_BUS': '/dev/i2c-1',
        'SWARM_KEY_URI': 'ecc://i2c-1:96?slot=0',
        'RESET': 60,
        'MAC': 'wlan0',
        'STATUS': 31,
        'BUTTON': 30,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False,
        'FCC_IDS': [],
        'CONTAINS_FCC_IDS': [],
        'IC_IDS': [],
        'CONTAINS_IC_IDS': []
        },

    # Nebra Hotspot HAT ROCK Pi 4 Indoor
    'nebra-indoor2': {
        'FRIENDLY': 'Nebra ROCK Pi 4 Indoor',
        'SUPPORTED_MODELS': ['Nebra ROCK Pi 4 Indoor'],
        'CPU_ARCH': 'aarch64',
        'BALENA_DEVICE_TYPE': ['rockpi-4b-rk3399'],
        'SPIBUS': 'spidev32766.0',
        'KEY_STORAGE_BUS': '/dev/i2c-7',
        'SWARM_KEY_URI': 'ecc://i2c-7:96?slot=0',
        'RESET': 149,
        'MAC': 'eth0',
        'STATUS': 156,
        'BUTTON': 154,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False,
        'GPIO_PIN_LED': 18,
        'GPIO_PIN_BUTTON': 16,
        'FCC_IDS': ['2AZDM-HHRK4'],
        'CONTAINS_FCC_IDS': ['2ARPP-GL5712UX', '2A3PA-ROCKPI4'],
        'IC_IDS': ['27187-HHRK4'],
        'CONTAINS_IC_IDS': []
        },

    # Nebra Hotspot HAT Tinkerboard 1
    'NEBHNT-HHTK': {
        'FRIENDLY': 'Nebra Hotspot HAT Tinkerboard Light',
        'SUPPORTED_MODELS': ['Nebra Hotspot HAT Tinkerboard Light'],
        'CPU_ARCH': 'armv7hf',
        'BALENA_DEVICE_TYPE': ['asus-tinker-board'],
        'SPIBUS': 'spidev2.0',
        'KEY_STORAGE_BUS': '/dev/i2c-1',
        'SWARM_KEY_URI': 'ecc://i2c-1:96?slot=0',
        'RESET': 167,
        'MAC': 'eth0',
        'STATUS': 163,
        'BUTTON': 162,
        'ECCOB': True,
        'TYPE': 'Light',
        'CELLULAR': False,
        'FCC_IDS': [],
        'CONTAINS_FCC_IDS': [],
        'IC_IDS': [],
        'CONTAINS_IC_IDS': []
        },

    # RAKwireless Hotspot Miner
    'rak-fl1': {
        'FRIENDLY': 'RAK Hotspot',
        'SUPPORTED_MODELS': ['RAK v1.5', 'RAK v2', 'RAK MNTD'],
        'CPU_ARCH': 'aarch64',
        'BALENA_DEVICE_TYPE': ['raspberrypi4-64'],
        'SPIBUS': 'spidev0.0',
        'KEY_STORAGE_BUS': '/dev/i2c-1',
        'SWARM_KEY_URI': 'ecc://i2c-1:96?slot=0',
        'RESET': 25,
        'MAC': 'wlan0',
        'STATUS': 20,
        'BUTTON': 7,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False,
        'FCC_IDS': [],
        'CONTAINS_FCC_IDS': [],
        'IC_IDS': [],
        'CONTAINS_IC_IDS': []
        },

    # Helium Hotspot
    'helium-fl1': {
        'FRIENDLY': 'Helium Hotspot',
        'SUPPORTED_MODELS': ['Helium Hotspot'],
        'CPU_ARCH': 'aarch64',
        'BALENA_DEVICE_TYPE': ['raspberrypi3-64', 'raspberrypi4-64'],
        'SPIBUS': 'spidev0.0',
        'KEY_STORAGE_BUS': '/dev/i2c-1',
        'SWARM_KEY_URI': 'ecc://i2c-1:96?slot=0',
        'RESET': 25,
        'MAC': 'wlan0',
        'STATUS': 20,
        'BUTTON': 7,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False,
        'FCC_IDS': [],
        'CONTAINS_FCC_IDS': [],
        'IC_IDS': [],
        'CONTAINS_IC_IDS': []
        },

    # SenseCAP M1 Hotspot
    'senscap-fl1': {
        'FRIENDLY': 'SenseCAP M1',
        'SUPPORTED_MODELS': ['SenseCAP M1'],
        'CPU_ARCH': 'aarch64',
        'BALENA_DEVICE_TYPE': ['raspberrypi4-64'],
        'SPIBUS': 'spidev0.0',
        'KEY_STORAGE_BUS': '/dev/i2c-1',
        'SWARM_KEY_URI': 'ecc://i2c-1:96?slot=0',
        'RESET': 17,
        'MAC': 'wlan0',
        'STATUS': 22,
        'BUTTON': 27,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False,
        'FCC_IDS': [],
        'CONTAINS_FCC_IDS': [],
        'IC_IDS': [],
        'CONTAINS_IC_IDS': []
        },

    # Panther X1
    'panther-fl1': {
        'FRIENDLY': 'Panther X1',
        'SUPPORTED_MODELS': ['Panther X1'],
        'CPU_ARCH': 'aarch64',
        'BALENA_DEVICE_TYPE': ['raspberrypi4-64'],
        'SPIBUS': 'spidev0.0',
        'KEY_STORAGE_BUS': '/dev/i2c-1',
        'SWARM_KEY_URI': 'ecc://i2c-1:96?slot=0',
        'RESET': 23,
        'MAC': 'wlan0',
        'STATUS': 22,
        'BUTTON': 21,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False,
        'FCC_IDS': [],
        'CONTAINS_FCC_IDS': [],
        'IC_IDS': [],
        'CONTAINS_IC_IDS': []
        },

    # Smart Mimic / Mimiq Finestra
    'finestra-fl1': {
        'FRIENDLY': 'Finestra Miner',
        'SUPPORTED_MODELS': ['Finestra Miner'],
        'CPU_ARCH': 'aarch64',
        'BALENA_DEVICE_TYPE': ['raspberrypi4-64'],
        'SPIBUS': 'spidev0.0',
        'KEY_STORAGE_BUS': '/dev/i2c-1',
        'SWARM_KEY_URI': 'ecc://i2c-1:96?slot=0',
        'RESET': 17,
        'MAC': 'wlan0',
        'STATUS_LED': {
            'TYPE': 'RGB',
            'GPIO_NUMBERS_RGB': [20, 26, 7],
            'GPIO_NUMBER_SINGLE': 20
         },
        'STATUS': 20,
        'BUTTON': 16,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False,
        'FCC_IDS': [],
        'CONTAINS_FCC_IDS': [],
        'IC_IDS': [],
        'CONTAINS_IC_IDS': []
        },

    # Pisces P100 Hotspot
    'pisces-fl1': {
        'FRIENDLY': 'Pisces P100',
        'SUPPORTED_MODELS': ['Pisces P100'],
        'CPU_ARCH': 'aarch64',
        'BALENA_DEVICE_TYPE': ['raspberrypi4-64'],
        'SPIBUS': 'spidev0.0',
        'KEY_STORAGE_BUS': '/dev/i2c-0',
        'SWARM_KEY_URI': 'ecc://i2c-0:96?slot=0',
        'RESET': 23,
        'MAC': 'eth0',
        'STATUS': 17,
        'BUTTON': 22,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False,
        'FCC_IDS': [],
        'CONTAINS_FCC_IDS': [],
        'IC_IDS': [],
        'CONTAINS_IC_IDS': []
        },

    # Embit / Controllino / Conelcom
    'controllino-fl1': {
        'FRIENDLY': 'Controllino  (Conelcom) Hotspot',
        'SUPPORTED_MODELS': ['Controllino Hotspot', 'Conelcom Hotspot'],
        'CPU_ARCH': 'aarch64',
        'BALENA_DEVICE_TYPE': ['raspberrypi4-64'],
        'SPIBUS': 'spidev0.0',
        'KEY_STORAGE_BUS': '/dev/i2c-1',
        'SWARM_KEY_URI': 'ecc://i2c-1:96?slot=0',
        'RESET': 4,
        'MAC': 'wlan0',
        'STATUS_LED': {
            'TYPE': 'RGB',
            'GPIO_NUMBERS_RGB': [20, 26, 7],
            'GPIO_NUMBER_SINGLE': 20
         },
        'STATUS': 20,
        'BUTTON': False,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False,
        'FCC_IDS': [],
        'CONTAINS_FCC_IDS': [],
        'IC_IDS': [],
        'CONTAINS_IC_IDS': []
        },

    # DIY Pi Supply Hotspot HAT
    'DIY-PISLGH': {
        'FRIENDLY': 'DIY Pi Supply Hotspot HAT',
        'SUPPORTED_MODELS': ['Raspberry Pi DIY Pi Supply Hotspot HAT'],
        'CPU_ARCH': '',
        'BALENA_DEVICE_TYPE': ['raspberry-pi', 'raspberry-pi2',
                               'raspberrypi0-2w-64', 'raspberrypi3-64',
                               'raspberrypi4-64'],
        'SPIBUS': 'spidev0.0',
        'KEY_STORAGE_BUS': '/dev/i2c-1',
        'SWARM_KEY_URI': 'ecc://i2c-1:96?slot=0',
        'RESET': 22,
        'MAC': 'eth0',
        'STATUS': 20,
        'BUTTON': 21,
        'ECCOB': False,
        'TYPE': 'Light',
        'CELLULAR': False,
        'FCC_IDS': [],
        'CONTAINS_FCC_IDS': [],
        'IC_IDS': [],
        'CONTAINS_IC_IDS': []
        },

    # Nebra Indoor Hotspot
    'DIY-RAK2287': {
        'FRIENDLY': 'DIY RAK2247/RAK2287 HAT',
        'SUPPORTED_MODELS': ['Raspberry Pi DIY RAK2247/RAK2287 HAT'],
        'CPU_ARCH': '',
        'BALENA_DEVICE_TYPE': ['raspberry-pi', 'raspberry-pi2',
                               'raspberrypi0-2w-64', 'raspberrypi3-64',
                               'raspberrypi4-64'],
        'SPIBUS': 'spidev0.0',
        'KEY_STORAGE_BUS': '/dev/i2c-1',
        'SWARM_KEY_URI': 'ecc://i2c-1:96?slot=0',
        'RESET': 17,
        'MAC': 'eth0',
        'STATUS': 20,
        'BUTTON': 21,
        'ECCOB': False,
        'TYPE': 'Light',
        'CELLULAR': False,
        'FCC_IDS': [],
        'CONTAINS_FCC_IDS': [],
        'IC_IDS': [],
        'CONTAINS_IC_IDS': []
        },

    # Linxdot CM4
    'linxdot-fl1': {
        'FRIENDLY': 'Linxdot Hotspot',
        'SUPPORTED_MODELS': ['Linxdot Hotspot'],
        'CPU_ARCH': 'aarch64',
        'BALENA_DEVICE_TYPE': ['raspberrypicm4-ioboard'],
        'SPIBUS': 'spidev0.0',
        'KEY_STORAGE_BUS': '/dev/i2c-1',
        'SWARM_KEY_URI': 'ecc://i2c-1:96?slot=0',
        'RESET': 17,
        'MAC': 'wlan0',
        'STATUS': 22,
        'BUTTON': 27,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False,
        'FCC_IDS': [],
        'CONTAINS_FCC_IDS': [],
        'IC_IDS': [],
        'CONTAINS_IC_IDS': []
        },

    # Pycom CM4
    'pycom-fl1': {
        'FRIENDLY': 'Pycom Hotspot',
        'SUPPORTED_MODELS': ['Pycom Hotspot'],
        'CPU_ARCH': 'aarch64',
        'BALENA_DEVICE_TYPE': ['raspberrypicm4-ioboard'],
        'SPIBUS': 'spidev0.0',
        'KEY_STORAGE_BUS': '/dev/i2c-10',
        'SWARM_KEY_URI': 'ecc://i2c-10:96?slot=0',
        'RESET': 23,
        'MAC': 'wlan0',
        'STATUS': 22,
        'BUTTON': 27,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False,
        'FCC_IDS': [],
        'CONTAINS_FCC_IDS': [],
        'IC_IDS': [],
        'CONTAINS_IC_IDS': []
        },

    # Syncrobit CM4
    'syncrobit-fl1': {
        'FRIENDLY': 'Syncrobit Hotspot',
        'SUPPORTED_MODELS': ['Syncrobit Hotspot'],
        'CPU_ARCH': 'aarch64',
        'BALENA_DEVICE_TYPE': ['raspberrypicm4-ioboard'],
        'SPIBUS': 'spidev0.0',
        'KEY_STORAGE_BUS': '/dev/i2c-1',
        'SWARM_KEY_URI': 'ecc://i2c-1:96?slot=0',
        'RESET': 17,
        'MAC': 'wlan0',
        'STATUS': 22,
        'BUTTON': 27,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False,
        'FCC_IDS': [],
        'CONTAINS_FCC_IDS': [],
        'IC_IDS': [],
        'CONTAINS_IC_IDS': []
        },

    # COTX X3 Hotspot
    'cotx-fl1': {
        'FRIENDLY': 'COTX X3',
        'SUPPORTED_MODELS': ['COTX X3'],
        'CPU_ARCH': 'aarch64',
        'BALENA_DEVICE_TYPE': ['raspberrypi4-64'],
        'SPIBUS': 'spidev0.0',  # There is a CSN1 pin which is connected to GPIO6 (HAT Pin 31)
        'KEY_STORAGE_BUS': '/dev/i2c-1',
        'SWARM_KEY_URI': 'ecc://i2c-1:96?slot=0',
        'RESET': 22,
        'MAC': 'eth0',
        'STATUS': 21,  # Stub. There is no status LED on X3. I2C-3 is used for display
                       # communication (HAT pins 5,7)
        'BUTTON': 23,
        'ECCOB': True,
        'TYPE': 'Full',
        'CELLULAR': False,  # There is a 4G option on the HAT board.
        'FCC_IDS': [],
        'CONTAINS_FCC_IDS': [],
        'IC_IDS': [],
        'CONTAINS_IC_IDS': []
        }
}

# Note: Maintain old names for backward compatibility, should be removed at some
# point of time.
variant_definitions['Indoor'] = variant_definitions['nebra-indoor1']
variant_definitions['NEBHNT-IN1'] = variant_definitions['nebra-indoor1']
variant_definitions['NEBHNT-OUT1'] = variant_definitions['nebra-outdoor1']
variant_definitions['Outdoor'] = variant_definitions['nebra-outdoor1']
variant_definitions['NEBHNT-LGT-ZS'] = variant_definitions['nebra-light1']
variant_definitions['NEBHNT-LGT-RADXA'] = variant_definitions['nebra-light2']
variant_definitions['NEBHNT-HHRK4'] = variant_definitions['nebra-indoor2']
variant_definitions['NEBHNT-HHRK4-OUT'] = variant_definitions['nebra-outdoor2']
variant_definitions['COMP-RAKHM'] = variant_definitions['rak-fl1']
variant_definitions['COMP-HELIUM'] = variant_definitions['helium-fl1']
variant_definitions['COMP-SENSECAPM1'] = variant_definitions['senscap-fl1']
variant_definitions['COMP-PANTHERX1'] = variant_definitions['panther-fl1']
variant_definitions['COMP-FINESTRA'] = variant_definitions['finestra-fl1']
variant_definitions['COMP-PISCESP100'] = variant_definitions['pisces-fl1']
variant_definitions['COMP-COTX3'] = variant_definitions['cotx-fl1']
variant_definitions['COMP-CONTROLLINO'] = variant_definitions['controllino-fl1']
variant_definitions['COMP-LINXDOTCM4'] = variant_definitions['linxdot-fl1']
variant_definitions['COMP-PYCOM'] = variant_definitions['pycom-fl1']
variant_definitions['COMP-SYNCROBITCM4'] = variant_definitions['syncrobit-fl1']


def get_variant_attribute(variant_name, attribute_key):
    """
    Returns the value of an attribute from a specific variant.
    Raises UnknownVariantException and UnknownVariantAttributeException.
    """

    try:
        variant_dict = variant_definitions[variant_name]
    except KeyError:
        raise UnknownVariantException("Variant %s is not recognized."
                                      % variant_name)

    try:
        return variant_dict[attribute_key]
    except KeyError:
        raise UnknownVariantAttributeException("Variant attribute %s"
                                               " is not recognized."
                                               % attribute_key)



# flake8: noqa
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: blockchain_txn.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import hm_pyhelper.protos.blockchain_txn_add_gateway_v1_pb2 as blockchain__txn__add__gateway__v1__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x14\x62lockchain_txn.proto\x12\x06helium\x1a#blockchain_txn_add_gateway_v1.proto\"U\n\x0e\x62lockchain_txn\x12<\n\x0b\x61\x64\x64_gateway\x18\x01 \x01(\x0b\x32%.helium.blockchain_txn_add_gateway_v1H\x00\x42\x05\n\x03txnb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'blockchain_txn_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _BLOCKCHAIN_TXN._serialized_start=69
  _BLOCKCHAIN_TXN._serialized_end=154
# @@protoc_insertion_point(module_scope)
